# -*- coding: utf-8 -*-

# This file is part of EUhu.
#
# EUhu is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# EUhu is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
#

import datetime


'''
This file contains common and helper mapping functions used by one or
more programs.
'''


def age(when, on=None):
    """Returns the age in years from the given date "when" since today. If
    "on" is provided the age will be calculated relative to the given
    date. See http://keyes.ie/calculate-age-with-python/

    :when: datetime object
    :on: datetime object
    :returns: integer

    """
    if on is None:
        on = datetime.today()
    was_earlier = (on.month, on.day) < (when.month, when.day)
    return on.year - when.year - (was_earlier)


def handle_single_field_transfer(program, entity, bmas_number):
    bmas = {}
    if str(program.get(entity)) == "0":
        bmas[bmas_number] = False
    elif str(program.get(entity)) == "1":
        bmas[bmas_number] = True
    return bmas

def handle_student(program, entity, bmas_number):
    bmas = {}
    if str(program.get(entity)) == "0":
        bmas[bmas_number] = False
    elif str(program.get(entity)) == "1":
        bmas[bmas_number] = True
    elif str(program.get(entity)) == "2":
        bmas[bmas_number] = False
    return bmas

def handle_employment(program, entity, bmas_number):
    bmas = {}
    if str(program.get(entity)) == "0":
        bmas[bmas_number] = False
    elif str(program.get(entity)) == "1":
        bmas[bmas_number] = True
    elif str(program.get(entity)) == "2":
        bmas[bmas_number] = True
    return bmas


def handle_gender(program):
    bmas = {}
    if str(program.get("gender")) == "0":
        bmas["7"] = "w"
    elif str(program.get("gender")) == "1":
        bmas["7"] = "m"
    return bmas


def handle_birthday(program):
    bmas = {}
    if (program.get("birthday") is not None
        and program.get("start_date") is not None):
        bmas = {"8": False, "9": False, "10": False}
        entry_age = age(program.get("birthday"), program.get("start_date"))
        if entry_age < 25:
            bmas["8"] = True
        if entry_age < 27:
            bmas["9"] = True
        if entry_age > 54:
            bmas["10"] = True
    return bmas


def handle_nr_field(program, entity, bmas_number, entity_value):
    bmas = {}
    if program.get(entity) is not None:
        bmas[bmas_number] = (str(program.get(entity)) == entity_value)
    return bmas


def handle_lm_employment_supportpurchased(program):
    bmas = {}
    if program.get("lm_employment_supportpurchased") is not None:
        bmas = {"11.1": False, "11.2": False, "11.3": False}
        value = str(program.get("lm_employment_supportpurchased"))
        if value == "0":
            bmas["11.1"] = True
        elif value == "1":
            bmas["11.2"] = True
        elif value == "2":
            bmas["11.3"] = True
        elif value == "3":
            bmas["11.2"] = True
            bmas["11.3"] = True
    return bmas


def handle_lm_employment_category_employed(program):
    bmas = {}
    bmas.update(handle_employment(program, "lm_employment_category_employed", "14.1"))
    bmas.update(handle_student(program, "lm_employment_category_fulltime_student", "14.7"))
    for entity, bmas_number in (
            ("lm_employment_category_underemployed", "14.2"),
            ("lm_employment_category_selfemployed", "14.3"),
            ("lm_employment_category_school", "14.4"),
            ("lm_employment_category_education_enterprise", "14.5"),
            ("lm_employment_category_education_school", "14.6"),
            ("lm_employment_category_training", "14.8"),
            ("lm_employment_category_inactive", "14.9")):
        bmas.update(handle_single_field_transfer(program, entity, bmas_number))
    return bmas


def handle_edu_attainment_vocational(program):
    bmas = {}
    for entity, bmas_number, entity_value in (
            ("edu_attainment_vocational", "16.1", "1"),
            ("edu_attainment_vocational", "16.2", "2"),
            ("edu_attainment_vocational", "16.3", "3"),
            ("edu_attainment_vocational", "16.4", "4"),
            ("edu_attainment_vocational", "16.5", "0")):
        bmas.update(handle_nr_field(program, entity, bmas_number, entity_value))
    return bmas


def handle_date(data, fieldname, bmas_nr):
    bmas = {}
    if data.get(fieldname) is not None:
        bmas[bmas_nr] = data.get(fieldname)
    return bmas
