# -*- coding: utf-8 -*-

# This file is part of EUhu.
#
# EUhu is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# EUhu is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License

from euhu.programs.base import to_bmas as sib_to_bmas
from euhu.validationresult import ValidationResult
from euhu.esf import to_esf
from euhu.esf_rules import validate_rules
from euhu.programs import handle_single_field_transfer, handle_nr_field
from euhu.programs.pwe import handle_PWE1


def to_bmas(data):
    '''
    Convert SIB input data to BMAS format.
    '''
    sib = sib_to_bmas(data)
    sib.update(handle_sib_edu_attaiment_school(data))
    sib.update(handle_single_field_transfer(data, "change_to_si", "34"))
    sib.update(handle_PWE1(sib))
    return sib


def validate(values):
    """Will return True if the submitted values meet the requirements
    for an evaluation of the EU output und result indicators. Else False.

    The function will first convert the WASKIQ specific values into the
    BMAS format. The BMAS values will be translated into the EU formar.
    The EU format will be validated.

    :values: Dictionary of values to be validated.
    :returns: True or False

    """
    bmas = to_bmas(values)
    eu = to_esf(bmas, {})
    rules = validate_rules(bmas, eu.esf)
    return ValidationResult(rules, eu)


def handle_sib_edu_attaiment_school(data):
    bmas = {}
    for entity, bmas_number, entity_value in (
            ("edu_attainment_school", "15.1", "2"),
            ("edu_attainment_school", "15.2", "3"),
            ("edu_attainment_school", "15.3", "4"),
            ("edu_attainment_school", "15.4", "5"),
            ("edu_attainment_school", "15.5", "6"),
            ("edu_attainment_school", "15.6a", "7"),
            ("edu_attainment_school", "15.6b", "8"),
            ("edu_attainment_school", "15.7b", "1")):
        bmas.update(handle_nr_field(data, entity, bmas_number,
                                    entity_value))
    # See issue784 for more details. Value 10 must also map to
    # 15.7a.
    if data.get(entity) is not None:
        if str(data.get(entity)) in ["0", "9"]:
            bmas["15.7a"] = True
        else:
            bmas["15.7a"] = False
    return bmas
