# -*- coding: utf-8 -*-
#
# This file is part of EUhu.
#
# EUhu is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# EUhu is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
#
# Authors: Ludwig Reiter <ludwig.reiter@intevation.de>
#
from collections import namedtuple

Indicator = namedtuple('Indicator', 'name description value')

Rule = namedtuple('Rule', 'name indicator_names success error category')


class ValidationResult(object):
    """Interface to the result of and indicator validation."""

    rules = {}

    indicator_labels = {
        "CO01": u"Arbeitslose, auch Langzeitarbeitslose",
        "CO02": u"Langzeitarbeitslose",
        "CO03": u"Nichterwerbstätige",
        "CO04": u"Nichterwerbstätige, die keine schulische oder berufliche Bildung absolvieren",
        "CO05": u"Erwerbstätige, auch Selbständige",
        "CO06": u"Unter 25-Jährige",
        "CO07": u"Über 54-Jährige",
        "CO08": u"Über 54-Jährige, die arbeitslos sind, einschließlich Langzeitarbeitsloser, oder die nicht erwerbstätig sind und keine schulische oder berufliche Bildung absolvieren",
        "CO09": u"Mit Grundbildung (ISCED 1) oder Sekundarbildung Unterstufe (ISCED 2)",
        "CO10": u"Mit Sekundarbildung Oberstufe (ISCED 3) oder postsekundärer Bildung (ISCED 4)",
        "CO11": u"Mit tertiärer Bildung (ISCED 5 bis 8)",
        "CO15": u"Migranten, Teilnehmende ausländischer Herkunft, Angehörige von Minderheiten (u.a. marginalisierte Gemeinschaften, wie etwa die Roma)",
        "CO16": u"Teilnehmende mit Behinderungen",
        "CO17": u"Sonstige benachteiligte Personen",
        "CO18": u"Obdachlose oder von Ausgrenzung auf dem Wohnungsmarkt Betroffene",
        "CR01": u"Nichterwerbstätige Teilnehmende, die nach ihrer Teilnahme auf Arbeitsuche sind",
        "CR02": u"Teilnehmende, die nach ihrer Teilnahme eine schulische/berufliche Bildung absolvieren",
        "CR03": u"Teilnehmende, die nach ihrer Teilnahme eine Qualifizierung erlangen",
        "CR04": u"Teilnehmende, die nach ihrer Teilnahme einen Arbeitsplatz haben, einschließlich Selbständige",
        "CR05": u"Benachteiligte Teilnehmende, die nach ihrer Teilnahme auf Arbeitsuche sind, eine schulische/berufliche Bildung absolvieren, eine Qualifizierung erlangen, einen Arbeitsplatz haben, einschließlich Selbständige",
        "PO04": u"Benachteiligte junge Menschen U27",
        "A2.1": u"Teilnehmende, die nach ihrer Teilnahme einen Arbeitsplatz haben, auf Arbeitsuche sind oder für den Arbeitsmarkt aktiviert wurden",
        "A3.1": u"Teilnehmende an Qualifizierungsmaßnahmen, die volle Gleichwertigkeit ihrer Abschlüsse erreichen bzw. die Voraussetzungen für eine qualifikationsadäquate Einmündung in den Arbeitsmarkt erfüllen",
        "PWE_GENDER": u"Männlicher Teilnehmer",
    }

    indicator_descriptions = {
        "CO01": u"eine/n arbeitslose/n",
        "CO02": u"eine/n langzeitarbeitslose/n",
        "CO03": u"eine/n nichterwerbstätige/n",
        "CO04": u"eine/n nichterwerbstätige/n Teilnehmende/n, der/die keine schulische oder berufliche Bildung absolviert",
        "CO05": u"eine/n erwerbstätige/n (auch selbständige/n)",
        "CO06": u"eine/n unter 25-Jährige/n",
        "CO07": u"eine/n über 54-Jährige/n",
        "CO08": u"eine/n über 54-Jährige/n, der arbeitslos ist, einschließlich Langzeitarbeitsloser, oder der/die nicht erwerbstätig ist und keine schulische oder berufliche Bildung absolviert",
        "CO09": u"eine/n Teilnehmende/n mit Grundbildung (ISCED 1) oder Sekundarbildung Unterstufe (ISCED 2)",
        "CO10": u"eine/n Teilnehmende/n mit Sekundarbildung Oberstufe (ISCED 3) oder postsekundärer Bildung (ISCED 4)",
        "CO11": u"eine/n Teilnehmende/n mit tertiärer Bildung (ISCED 5 bis 8)",
        "CO15": u"eine/n Migranten/in, Teilnehmende/n ausländischer Herkunft, Angehörige/n von Minderheiten (u.a. marginalisierte Gemeinschaften, wie etwa die Roma)",
        "CO16": u"eine/n Teilnehmende/n mit Behinderungen",
        "CO17": u"eine sonstig benachteiligte Person",
        "CO18": u"eine/n Obdachlose/n oder von Ausgrenzung auf dem Wohnungsmarkt Betroffene/n",
        "CR01": u"eine/n nichterwerbstätige/n Teilnehmende/n, der/die nach seiner/ihrer Teilnahme auf Arbeitsuche ist",
        "CR02": u"eine/n Teilnehmende/n, der/die nach seine/ihrer Teilnahme eine schulische/berufliche Bildung absolviert",
        "CR03": u"eine/n Teilnehmende/n, der/die nach seine/ihrer Teilnahme eine Qualifizierung erlangt",
        "CR04": u"eine/n Teilnehmende/n, der/die nach seine/ihrer Teilnahme einen Arbeitsplatz hat (auch Selbständige)",
        "CR05": u"eine/n benachteiligte Teilnehmende/n, der/die nach seiner/ihrer Teilnahme auf Arbeitsuche ist, eine schulische/berufliche Bildung absolviert, eine Qualifizierung erlangt, einen Arbeitsplatz hat (auch Selbständige)",
        "PO04": u"einen benachteiligten jungen Menschen (U27)",
        "A2.1": u"eine/n Teilnehmende/n, der/die nach seiner/ihrer Teilnahme einen Arbeitsplatz hat, auf Arbeitsuche ist oder für den Arbeitsmarkt aktiviert wurde",
        "A3.1": u"eine/n Teilnehmende/n an Qualifizierungsmaßnahmen, der/die volle Gleichwertigkeit seiner/ihrer Abschlüsse erreicht bzw. die Voraussetzungen für eine qualifikationsadäquate Einmündung in den Arbeitsmarkt erfüllt",

    }

    def __init__(self, validation_result, eu_indicators):
        self.result = validation_result
        self.eu_indicators = eu_indicators
        self.violated = []
        self.status = {}
        rule2 = Rule("R2", ["CO02"], u"Die Angaben zu Arbeitslosigkeit und Langzeitarbeitslosigkeit sind schlüssig.",
                     u"Die Angaben zu Arbeitslosigkeit und Langzeitarbeitslosigkeit sind nicht schlüssig.",
                     "Erwerbstatus")
        rule3 = Rule("R3", ["CO04"], u"Die Angaben zur Nichterwerbstätigkeit sind plausibel.",
                     u"Die Angaben zur Nichterwerbstätigkeit sind nicht plausibel.", "Erwerbstatus")
        rule4 = Rule("R4", ["CO06", "CO07"],
                     u"Die Angaben zum Alter des/der Teilnehmenden kann einer eindeutigen Altersklasse zugeordnet werden.",
                     u"Ihre bisherigen Angaben lassen zugleich auf {} schließen", "Stammdaten")
        rule5 = Rule("R5", ["CO09", "CO10", "CO11"],
                     u"Die Angaben zum Berufs-/Bildungsabschluss lassen sich eindeutig einer ISCED-Stufe zuordnen.",
                     u"Ihre bisherigen Angaben lassen zugleich auf {} schließen", "Bildungs- & Berufsabschluss")


        self.evaluate_rule1("CO01", "CO03", "CO05")
        self.evaluate_single_indicator_rule(rule2)
        self.evaluate_single_indicator_rule(rule3)
        self.evaluate_rule(rule4)
        self.evaluate_rule(rule5)
        if "PWE_GENDER" in validation_result:
            rule6 = Rule("RPWE", ["PWE_GENDER"], u"Weibliche Teilnehmende sind förderfähig.", u"Männliche Teilnehmende sind nicht förderfähig.", "Stammdaten")
            self.evaluate_rule(rule6)

    def evaluate_rule(self, rule):
        indicator_names = rule.indicator_names
        text = rule.success
        values = [self.result.get(x) for x in indicator_names]
        status = True
        if self.not_evaluable(*indicator_names):
            status = None
        elif not all(values):
            results = [self.indicator_descriptions[indicator]
                       for indicator in indicator_names if
                       self.eu_indicators.esf.get(indicator)]
            text = rule.error.format(' und '.join(results))
            status = False
            self.violated.append([text, rule.category])
        self.rules[rule.name] = [text, rule.category]
        self.status[rule.name] = status

    def evaluate_rule1(self, *indicator_names):
        text = (u"Die Angaben in der Teilnahme lassen eindeutig auf den "
                u"Erwerbstatus schließen.")
        values = [self.result.get(x) for x in indicator_names]
        status = True
        if self.not_evaluable(*indicator_names):
            status = None
        elif not all(values):
            results = [self.indicator_descriptions[indicator]
                       for indicator in indicator_names if
                       self.eu_indicators.esf.get(indicator)]
            failing = [self.eu_indicators.esf.get(x) for x in indicator_names]
            if not any(failing):
                text = (u"Ihre Angaben lassen keinen eindeutigen Schluss auf "
                        u"den Erwerbsstatus zu.")
            else:
                text = (u"Ihre bisherigen Angaben lassen zugleich auf {}"
                        u" Teilnehmende/n schließen.").format(
                    ' und '.join(results))
            status = False
            self.violated.append([text, "Erwerbstatus"])
        self.rules["R1"] = [text, "Erwerbstatus"]
        self.status["R1"] = status

    def evaluate_single_indicator_rule(self, rule):
        text = rule.success
        indicator_names = rule.indicator_names
        status = True
        values = [self.result.get(x) for x in indicator_names]
        if self.not_evaluable(*indicator_names):
            status = None
        elif not all(values):
            text = rule.error
            status = False
            self.violated.append([text, rule.category])
        self.rules[rule.name] = [text, rule.category]
        self.status[rule.name] = status

    def not_evaluable(self, *indicator_names):
        for indicator_name in indicator_names:
            if self.result.get(indicator_name) is None:
                return True
        return False

    def has_violated_rules(self):
        return len(self.violated) > 0

    def get_rule_description(self, rule):
        return self.rules[rule]

    def is_evaluable(self):
        return self.result.get("OKAY")

    def translate(self, value):
        if value is None:
            return "K.A."
        return "Ja" if value else "Nein"

    def get_indicators_by_list(self, indicator_names):
        return [Indicator(indicator, self.indicator_labels[indicator],
                self.translate(self.eu_indicators.esf.get(indicator)))
                for indicator in indicator_names]

    @property
    def indicators(self):
        names = ["CO01", "CO02", "CO03", "CO04",
                 "CO05", "CO06", "CO07", "CO08",
                 "CO09", "CO10", "CO11",
                 "CO15", "CO16",
                 "CO17", "CO18", "CR01", "CR02",
                 "CR03", "CR04", "CR05", "PO04",
                 "A2.1", "A3.1"]
        return self.get_indicators_by_list(names)

    @property
    def oi_indicators(self):
        names = ["CO01", "CO02", "CO03", "CO04",
                 "CO05", "CO06", "CO07", "CO08",
                 "CO09", "CO10", "CO11",
                 "CO15", "CO16",
                 "CO17", "CO18"]
        return self.get_indicators_by_list(names)

    @property
    def ri_indicators(self):
        names = ["CR01", "CR02", "CR03", "CR04", "CR05"]
        return self.get_indicators_by_list(names)

    @property
    def program_indicators(self):
        names = ["PO04", "A2.1", "A3.1"]
        return self.get_indicators_by_list(names)

    def set_ri_indicator(self, name, value):
        self.eu_indicators.esf[name] = value

    def get_ri_indicator(self, name):
        return self.eu_indicators.esf.get(name)
